"use strict";

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Player = function () {
    function Player(game, spawnPoint) {
        var _this = this;

        _classCallCheck(this, Player);

        if (!spawnPoint) {
            spawnPoint = new BABYLON.Vector3(0, 10, -10);
        }

        this.spawnPoint = spawnPoint;
        this.scene = game.scene;
        this.game = game;
        this.height = 2;
        this.speed = 1;
        this.inertia = 0.9;
        this.angularInertia = 0;
        // The mouse sensibility (lower the better sensible)
        this.angularSensibility = 1000;
        this.camera = this._initCamera();
        // The player must click on the canvas to activate control
        this.controlEnabled = false;
        this.weapon = new Weapon(game, this);

        var canvas = this.scene.getEngine().getRenderingCanvas();
        // Event listener on click on the canvas
        canvas.addEventListener("click", function (evt) {
            var width = _this.scene.getEngine().getRenderWidth();
            var height = _this.scene.getEngine().getRenderHeight();

            if (_this.controlEnabled) {
                var pickInfo = _this.scene.pick(width / 2, height / 2, null, false, _this.camera);
                _this.handleUserMouse(evt, pickInfo);
            }
        }, false);

        // Event listener to go pointer lock
        this._initPointerLock();

        // The representation of player in the minimap
        var s = BABYLON.Mesh.CreateSphere("player2", 16, 4, this.scene);
        s.position.y = 10;
        s.registerBeforeRender(function () {
            s.position.x = _this.camera.position.x;
            s.position.z = _this.camera.position.z;
        });

        var red = new BABYLON.StandardMaterial("red", this.scene);
        red.diffuseColor = BABYLON.Color3.Red();
        red.specularColor = BABYLON.Color3.Black();
        s.material = red;
        s.layerMask = 1;

        // Set the active camera for the minimap
        this.scene.activeCameras.push(this.camera);
        this.scene.activeCamera = this.camera;
    }

    _createClass(Player, [{
        key: "_initPointerLock",
        value: function _initPointerLock() {
            var _this2 = this;

            // Request pointer lock
            var canvas = this.scene.getEngine().getRenderingCanvas();
            canvas.addEventListener("click", function (evt) {
                canvas.requestPointerLock = canvas.requestPointerLock || canvas.msRequestPointerLock || canvas.mozRequestPointerLock || canvas.webkitRequestPointerLock;
                if (canvas.requestPointerLock) {
                    canvas.requestPointerLock();
                }
            }, false);

            // Event listener when the pointerlock is updated.
            var pointerlockchange = function pointerlockchange(event) {
                _this2.controlEnabled = document.mozPointerLockElement === canvas || document.webkitPointerLockElement === canvas || document.msPointerLockElement === canvas || document.pointerLockElement === canvas;
                if (!_this2.controlEnabled) {
                    _this2.camera.detachControl(canvas);
                } else {
                    _this2.camera.attachControl(canvas);
                }
            };
            document.addEventListener("pointerlockchange", pointerlockchange, false);
            document.addEventListener("mspointerlockchange", pointerlockchange, false);
            document.addEventListener("mozpointerlockchange", pointerlockchange, false);
            document.addEventListener("webkitpointerlockchange", pointerlockchange, false);
        }

        /**
         * Init the player camera
         * @returns {BABYLON.FreeCamera}
         * @private
         */

    }, {
        key: "_initCamera",
        value: function _initCamera() {

            var cam = new BABYLON.FreeCamera("camera", this.spawnPoint, this.scene);
            cam.attachControl(this.scene.getEngine().getRenderingCanvas());
            cam.ellipsoid = new BABYLON.Vector3(2, this.height, 2);
            cam.checkCollisions = true;
            cam.applyGravity = true;
            // WASD
            cam.keysUp = [87]; // W
            cam.keysDown = [83]; // S
            cam.keysLeft = [65]; // A
            cam.keysRight = [68]; // D
            cam.speed = this.speed;
            cam.inertia = this.inertia;
            cam.angularInertia = this.angularInertia;
            cam.angularSensibility = this.angularSensibility;
            cam.layerMask = 2;

            return cam;
        }

        /**
         * Handle the user input on keyboard
         * @param keycode
         */

    }, {
        key: "handleUserKeyboard",
        value: function handleUserKeyboard(keycode) {
            switch (keycode) {}
        }

        /**
         * Handle the user input on mouse.
         * click = shoot
         * @param evt
         * @param pickInfo The pick data retrieved when the click has been done
         */

    }, {
        key: "handleUserMouse",
        value: function handleUserMouse(evt, pickInfo) {
            this.weapon.fire(pickInfo);
        }
    }]);

    return Player;
}();

;



function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var PaintingEgg = function (_BABYLON$Mesh) {
    _inherits(PaintingEgg, _BABYLON$Mesh);

    function PaintingEgg(game, painting, posX, posZ) {
        _classCallCheck(this, PaintingEgg);

        //BABYLON.Mesh.call(this, "target", game.scene);

        var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(PaintingEgg).call(this, "target", game.scene));

        _this.game = game;
        _this.painting = painting;

        var vd = BABYLON.VertexData.CreateSphere(48, 39);
        vd.applyToMesh(_this, false);

        var m = new BABYLON.StandardMaterial(painting.name, game.scene);
        m.diffuseTexture = new BABYLON.Texture('assets/paintings/' + _this.assetName(painting.name) + '.jpg', game.scene);
        _this.material = m;

        _this.position = new BABYLON.Vector3(posX, 34, posZ);

        // Check collisions
        _this.checkCollisions = true;
        var u = 0;

        _this.game.scene.registerBeforeRender(function () {
            // this.rotation.y += Math.random() * 4 - 8;
            if (_this.human === null) {
                _this.rotation.x += Math.random() * 0.08;
            } else {
                // flying away
                _this.rotation.x += Math.random() * 0.12;
                _this.position.y += 0.05 + u * 0.02;
                u += 1;
            }
        });
        _this.human = null;
        return _this;
    }

    _createClass(PaintingEgg, [{
        key: 'explode',
        value: function explode() {
            var _this2 = this;

            if (this.human === null) {
                this.human = this.game.human = new PaintingHuman(this.game, this.painting, this.position.x + 12, this.position.z + 12);
                this.game.eggsCracked += 1;

                var cam = this.game.scene.activeCameras[0];
                var x = undefined,
                    z = undefined;
                if (cam.position.x <= this.human.position.x) {
                    x = this.game.size - 200;
                } else {
                    x = 82;
                }
                if (cam.position.z <= this.human.position.z) {
                    z = this.game.size - 200;
                } else {
                    z = 60;
                }

                this.game.wall = new Wall(this.game, this.painting, x, z);

                this.human.begin = Date.now();
                this.human.help = 0;
                this.human.memory = 100;
                this.human.xDistance = (x - this.human.position.x) / 20;
                this.human.yDistance = (22 - this.human.position.y) / 20;
                this.human.zDistance = (z - this.human.position.z) / 20;

                setTimeout(function () {
                    return _this2.dispose();
                }, 4000);
            } else {
                console.log('COMICS', 'you shouldn\'t kill those who lay eggs');
                // var sm = new BABYLON.StandardMaterial(`skyBox_${this.painting.name}`, this.game.scene);
                this.game.skybox.material.reflectionTexture = new BABYLON.CubeTexture('assets/paintings/' + this.assetName(this.painting.name) + '_heaven', this.game.scene);
                // sm.diffuseTexture = new BABYLON.Texture(`assets/paintings/${this.painting.name}_heaven.jpg`, this.game.scene);
                // this.game.skybox.material = sm;
            }
        }
    }, {
        key: 'assetName',
        value: function assetName(name) {
            return name.replace(' ', '_');
        }
    }]);

    return PaintingEgg;
}(BABYLON.Mesh);

;



function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var PaintingHuman = function (_BABYLON$Mesh) {
    _inherits(PaintingHuman, _BABYLON$Mesh);

    function PaintingHuman(game, painting, posX, posZ) {
        _classCallCheck(this, PaintingHuman);

        var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(PaintingHuman).call(this, "target", game.scene));

        _this.game = game;
        _this.painting = painting;

        var vd = BABYLON.VertexData.CreateBox(9);
        vd.applyToMesh(_this, false);
        _this.scaling.x = 0.2;
        _this.scaling.y = 1.6;
        _this.scaling.z = 1.8;
        _this.rotation.x = 1.58;
        _this.showBoundingBox = false;
        //Define a material
        var ma = new BABYLON.StandardMaterial(painting.name, game.scene);
        // ma.emissiveColor = new BABYLON.Color3(0, 0, 1);
        // ma.opacityTexture = new BABYLON.Texture(`assets/paintings/${this.assetName(painting.name)}_human.jpg`, game.scene);
        ma.diffuseTexture = new BABYLON.Texture("assets/paintings/" + _this.assetName(painting.name) + "_human.jpg", game.scene);
        ma.diffuseTexture.hasAlpha = true;
        // ma.alpha = 0.9;
        ma.useAlphaFromDiffuseTexture = true;
        ma.backFaceCulling = false;
        var ba = new BABYLON.StandardMaterial("material1", game.scene);
        ba.diffuseColor = new BABYLON.Color3(1, 1, 1);

        var multi = new BABYLON.MultiMaterial("a", game.scene);
        multi.subMaterials = [ba, ba, ma, ma, ba, ba];

        var verticesCount = _this.getTotalVertices();
        _this.subMeshes = [new BABYLON.SubMesh(0, 0, verticesCount, 0, 6, _this), new BABYLON.SubMesh(1, 1, verticesCount, 6, 6, _this), new BABYLON.SubMesh(2, 2, verticesCount, 12, 6, _this), new BABYLON.SubMesh(3, 3, verticesCount, 18, 6, _this), new BABYLON.SubMesh(4, 4, verticesCount, 24, 6, _this), new BABYLON.SubMesh(5, 5, verticesCount, 30, 6, _this)];

        _this.material = multi;

        // this.material = m;

        _this.position = new BABYLON.Vector3(posX, 8, posZ);

        // Check collisions
        _this.checkCollisions = true;

        _this.rotations = [];
        for (var j = 0; j < 32; j++) {
            _this.rotations[j] = Math.random() * 8 - 4;
        }

        var h = 16;
        var period = 5;
        var z = 0;

        _this.check = setInterval(function () {
            var time = Date.now() - _this.begin;
            if (time > (_this.game.remainingTime + _this.help) * 1000) {
                window.simulator.end('no time');
            } else if (z % period == 0) {
                var left = _this.timeLeft(Date.now());
                console.log("Left " + left);
                if (_this.material.subMaterials[3]) {
                    _this.material.subMaterials[3].alpha = left / _this.game.remainingTime;
                    _this.material.subMaterials[4].alpha = left / _this.game.remainingTime;
                }
            }
            z += 1;
        }, 1000);

        _this.game.scene.registerBeforeRender(function () {
            if (h % 16 == 0) {
                // this.rotation.y += this.rotations[(h / 16 + 46)% 32];
                // this.rotation.x += this.rotations[(h / 16 + 2)% 32];
                _this.position.z += _this.rotations[(h / 16 + 46) % 32];
                _this.position.x += _this.rotations[(h / 16 + 46) % 32];
            }
            h += 1;
        });
        _this.human = null;

        //talk stuff

        // this.memory = 100;
        _this.lastHit = 0;
        return _this;
    }

    _createClass(PaintingHuman, [{
        key: "explode",
        value: function explode() {
            // if (this.lastHit +  this.game.memory.protectedTime * 1000 < Date.now()) { //ms
            this.position.x += this.xDistance;
            this.position.y += this.yDistance;
            this.position.z += this.zDistance;

            // this.memory -= (100 / this.game.memory.memento);
            this.lastHit = Date.now();

            if (this.timeLeft(Date.now()) < 0) {
                // this.target.material = this.target.xMaterial;
                //if hit
                this.dispose();
                window.simulator.end('memory lost');
            }
            // }
        }
    }, {
        key: "timeLeft",
        value: function timeLeft(time) {
            return -Math.floor(time / 1000 - this.begin / 1000 - (this.game.remainingTime + this.help));
        }
    }, {
        key: "assetName",
        value: function assetName(name) {
            return name.replace(' ', '_');
        }
    }]);

    return PaintingHuman;
}(BABYLON.Mesh);

;

function hallucinateLevel(game) {
    setInterval(function () {
        game.scene.activeCameras[0].rotation.y += Math.random() * 0.2 - 0.4;
        game.scene.activeCameras[0].rotation.z += Math.random() * 0.12 - 0.24;
    }, 2200);

    //async shaking the cam for realistic effect

    setInterval(function () {
        game.scene.activeCameras[0].rotation.x += Math.random() * 0.08 - 0.14;
    }, 2600);

    var flashbacks = [];
    for (var f = 0; f < 2; f++) {
        flashbacks.push(new BABYLON.StandardMaterial('hallucinate', game.scene));
        flashbacks[f].diffuseTexture = new BABYLON.Texture('assets/paintings/flashback_' + f + '.jpg', game.scene);
    }

    if (game.human) {
        //patch explode
        flashbacks.push(game.human.material);
        setInterval(function () {
            game.human.material = flashbacks[Math.floor(Math.random() * 3)];
        }, 2200);
    }
}



function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var MagritteMan = function (_BABYLON$Mesh) {
    _inherits(MagritteMan, _BABYLON$Mesh);

    function MagritteMan(kind, game) {
        _classCallCheck(this, MagritteMan);

        //BABYLON.Mesh.call(this, "target", game.scene);

        var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(MagritteMan).call(this, "target", game.scene));

        _this.game = game;
        _this.kind = kind;
        _this.health = 4;
        _this.speed = 0.4;

        var vd = BABYLON.VertexData.CreateCylinder(42, 20, 12);
        vd.applyToMesh(_this, false);

        var ma = new BABYLON.StandardMaterial('z_' + kind, game.scene);
        ma.diffuseTexture = new BABYLON.Texture('assets/paintings/magritte_' + kind + '.jpg', game.scene);
        _this.material = ma;

        _this.position = new BABYLON.Vector3(Math.random() * game.size, _this.kind === 'pigeon' ? 22 : 220, Math.random() * game.size);
        console.log(_this.position);

        // Check collisions
        _this.checkCollisions = true;

        _this.game.scene.registerBeforeRender(function () {
            _this.position.y += _this.speed * (_this.kind === 'pigeon' ? 1 : -1);
        });
        return _this;
    }

    _createClass(MagritteMan, [{
        key: 'explode',
        value: function explode() {
            if (this.game.human) {
                this.game.human.help += this.health;
            }
            this.dispose();
        }
    }]);

    return MagritteMan;
}(BABYLON.Mesh);

;



function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var Wall = function (_BABYLON$Mesh) {
    _inherits(Wall, _BABYLON$Mesh);

    function Wall(game, painting, posX, posZ) {
        _classCallCheck(this, Wall);

        //BABYLON.Mesh.call(this, "target", game.scene);

        var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(Wall).call(this, "target", game.scene));

        _this.game = game;
        _this.painting = painting;

        var vd = BABYLON.VertexData.CreateBox(350);
        vd.applyToMesh(_this, false);

        _this.scaling.y = 0.4;
        _this.scaling.z = 0.01;
        var ma = new BABYLON.StandardMaterial(painting.name, game.scene);
        ma.diffuseColor = new BABYLON.Color3(0, 1, 0);
        ma.alpha = 0.2;
        _this.material = ma;

        _this.position = new BABYLON.Vector3(posX, 22, posZ);

        // Check collisions
        _this.checkCollisions = true;

        // this.game.scene.registerBeforeRender(() => {
        //     this.rotation.y += Math.random() * 4 - 8;
        //     this.rotation.x += Math.random() * 4 - 8;
        // });
        _this.human = null;
        return _this;
    }

    _createClass(Wall, [{
        key: "explode",
        value: function explode(e) {
            if (this.game.human.intersectsMesh(this)) {
                clearInterval(this.game.human.check);
                this.game.human.dispose();
                this.game.human = null;

                var u = this.game.eggs[this.game.eggsCracked];
                this.game.eggs[this.game.eggsCracked] = new PaintingEgg(this.game, u.painting, u.posX, u.posZ);

                this.dispose();
            }
        }
    }]);

    return Wall;
}(BABYLON.Mesh);

;



function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var Target = function (_BABYLON$Mesh) {
    _inherits(Target, _BABYLON$Mesh);

    function Target(game, painting, posX, posZ) {
        _classCallCheck(this, Target);

        //BABYLON.Mesh.call(this, "target", game.scene);

        var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(Target).call(this, "target", game.scene));

        _this.game = game;
        _this.painting = painting;
        window.paintings.push(_this);

        // var vd = BABYLON.VertexData.CreateBox(1);
        // var vd = BABYLON.Mesh.CreateBox(painting.name, 28, game.scene);
        var vd = BABYLON.VertexData.CreateBox(18, 10, 18 * 2.2);
        // vd.scaling.x = 0.4;
        // vd.scaling.z = 0.22;
        vd.applyToMesh(_this, false);

        // debugger;
        var m = new BABYLON.StandardMaterial(painting.name, game.scene);
        m.diffuseTexture = new BABYLON.Texture('assets/paintings/' + _this.assetName(painting.name) + '.jpg', game.scene);
        _this.material = m;

        _this.position = new BABYLON.Vector3(posX, 8, posZ);

        // Check collisions
        _this.checkCollisions = true;

        _this.rotations = [];
        for (var j = 0; j < 32; j++) {
            _this.rotations[j] = Math.random() * 0.2 - 0.1;
        }

        var h = 16;
        _this.game.scene.registerBeforeRender(function () {
            if (h % 16 == 0) {
                _this.rotation.y += _this.rotations[(h / 16 + 46) % 32];
                // this.rotation.x += this.rotations[(h / 16 + 2)% 32];
            }
            h += 1;
        });
        _this.human = null;

        _this.memory = 100;
        _this.lastHit = 0;
        return _this;
    }

    _createClass(Target, [{
        key: 'explode',
        value: function explode() {
            // we become gray
            // the human goes out as a target
            // the heaven gets our background
            this.xMaterial = this.material;
            this.material = undefined;

            this.humans = [];
            for (var i = 0; i < 10; i++) {
                this.humans.push(new PaintingHuman(this.game, this.painting, this, Math.random() * 320 - 160, Math.random() * 400 - 200));
            }
            // let sm = new BABYLON.StandardMaterial(`skybox_${this.painting.name}`, this.game.scene);
            // this.game.skybox.material.reflectionTexture =
            var _iteratorNormalCompletion = true;
            var _didIteratorError = false;
            var _iteratorError = undefined;

            try {
                for (var _iterator = this.game.clouds[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                    var cloud = _step.value;

                    cloud.material.diffuseTexture = new BABYLON.Texture('assets/paintings/' + this.assetName(this.painting.name) + '_heaven.jpg', scene);
                }
                // new BABYLON.CubeTexture(`assets/paintings/${this.assetName(this.painting.name)}_heaven`, this.game.scene);
                // this.game.skybox.material = sm;
            } catch (err) {
                _didIteratorError = true;
                _iteratorError = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion && _iterator.return) {
                        _iterator.return();
                    }
                } finally {
                    if (_didIteratorError) {
                        throw _iteratorError;
                    }
                }
            }

            this.dispose();
        }
    }, {
        key: 'assetName',
        value: function assetName(name) {
            return name.replace(' ', '_');
        }
    }]);

    return Target;
}(BABYLON.Mesh);

;



var Arena = function () {
    function Arena(game) {
        var _this = this;

        _classCallCheck(this, Arena);

        this.game = game;

        // The ground
        var ground = BABYLON.Mesh.CreateGround("ground", this.game.size, this.game.size, 2, this.game.scene);
        var floorIsSlava = new BABYLON.StandardMaterial("floorIsSlava", this.game.scene);
        floorIsSlava.diffuseTexture = new BABYLON.Texture("assets/particles/floorIsSlava.jpg", this.game.scene);
        floorIsSlava.diffuseTexture.uScale = 5.0;
        floorIsSlava.diffuseTexture.vScale = 5.0;
        floorIsSlava.backFaceCulling = false;

        ground.material = floorIsSlava;
        this._deactivateSpecular(ground);
        ground.checkCollisions = true;

        this.game.eggs = [new PaintingEgg(this.game, this.game.paintings[0], this._randomNumber(-this.game.size / 2, this.game.size / 2), this._randomNumber(-this.game.size / 2, this.game.size / 2))].concat(['', '', '', ''].map(function (u, v) {
            return {
                painting: _this.game.paintings[v + 1],
                posX: _this._randomNumber(-_this.game.size / 2, _this.game.size / 2),
                posZ: _this._randomNumber(-_this.game.size / 2, _this.game.size / 2)
            };
        }));
        this.game.eggsCracked = 0;

        new MagritteMan('pigeon', this.game);
        for (var u = 0; u < 10; u++) {
            new MagritteMan('pigeon', this.game);
            new MagritteMan('apple', this.game);
        }
        setInterval(function () {
            new MagritteMan('pigeon', _this.game);
            new MagritteMan('apple', _this.game);
        }, 10000);

        var mm = new BABYLON.FreeCamera("minimap", new BABYLON.Vector3(0, 100, 0), this.game.scene);
        mm.layerMask = 1;
        mm.setTarget(new BABYLON.Vector3(0.1, 0.1, 0.1));
        mm.mode = BABYLON.Camera.ORTHOGRAPHIC_CAMERA;

        mm.orthoLeft = -this.game.size / 2;
        mm.orthoRight = this.game.size / 2;
        mm.orthoTop = this.game.size / 2;
        mm.orthoBottom = -this.game.size / 2;

        mm.rotation.x = Math.PI / 2;

        var xstart = 0.8,
            ystart = 0.75;
        var width = 0.99 - xstart,
            height = 1 - ystart;

        mm.viewport = new BABYLON.Viewport(xstart, ystart, width, height);
        this.game.scene.activeCameras.push(mm);
    }

    _createClass(Arena, [{
        key: "_randomNumber",

        /**
         * Generates a random number between min and max
         * @param min
         * @param max
         * @returns {number}
         * @private
         */
        value: function _randomNumber(min, max) {
            if (min == max) {
                return min;
            }
            var random = Math.random();
            return random * (max - min) + min;
        }
    }, {
        key: "_deactivateSpecular",
        value: function _deactivateSpecular(mesh) {
            if (!mesh.material) {
                mesh.material = new BABYLON.StandardMaterial(mesh.name + "mat", this.game.scene);
            }
            mesh.material.specularColor = BABYLON.Color3.Black();
        }
    }]);

    return Arena;
}();

;



var Weapon = function () {
    function Weapon(game, player) {
        var _this = this;

        _classCallCheck(this, Weapon);

        this.game = game;
        this.player = player;

        // The weapon mesh
        var wp = game.assets["gun"][0];
        wp.isVisible = true;
        wp.rotationQuaternion = null;
        wp.rotation.x = -Math.PI / 2;
        wp.rotation.y = Math.PI;
        wp.parent = player.camera;
        wp.position = new BABYLON.Vector3(0.25, -0.4, 1);
        this.mesh = wp;

        // The initial rotation
        this._initialRotation = this.mesh.rotation.clone();

        // The fire rate
        this.fireRate = 250.0;
        this._currentFireRate = this.fireRate;
        this.canFire = true;

        // The particle emitter
        var scene = this.game.scene;
        var particleSystem = new BABYLON.ParticleSystem("particles", 100, scene);
        particleSystem.emitter = this.mesh; // the starting object, the emitter
        particleSystem.particleTexture = new BABYLON.Texture("assets/particles/eraserhead.png", scene);
        particleSystem.emitRate = 2;
        particleSystem.blendMode = BABYLON.ParticleSystem.BLENDMODE_STANDARD;
        particleSystem.minEmitPower = 1;
        particleSystem.maxEmitPower = 3;
        particleSystem.colorDead = new BABYLON.Color4(1, 1, 1, 0.0);

        particleSystem.minLifeTime = 0.2;
        particleSystem.maxLifeTime = 0.2;

        particleSystem.updateSpeed = 0.02;
        //particleSystem.start();
        this.particleSystem = particleSystem;

        this.game.scene.registerBeforeRender(function () {
            if (!_this.canFire) {
                _this._currentFireRate -= BABYLON.Tools.GetDeltaTime();
                if (_this._currentFireRate <= 0) {
                    _this.canFire = true;
                    _this._currentFireRate = _this.fireRate;
                }
            }
        });
    }

    _createClass(Weapon, [{
        key: "animate",
        value: function animate() {
            var _this2 = this;

            this.particleSystem.start();

            var start = this._initialRotation.clone();
            var end = start.clone();

            end.x += Math.PI / 10;

            // Create the Animation object
            var display = new BABYLON.Animation("fire", "rotation", 60, BABYLON.Animation.ANIMATIONTYPE_VECTOR3, BABYLON.Animation.ANIMATIONLOOPMODE_CONSTANT);

            // Animations keys
            var keys = [{
                frame: 0,
                value: start
            }, {
                frame: 10,
                value: end
            }, {
                frame: 100,
                value: start
            }];

            // Add these keys to the animation
            display.setKeys(keys);

            // Link the animation to the mesh
            this.mesh.animations.push(display);

            // Run the animation !
            this.game.scene.beginAnimation(this.mesh, 0, 100, false, 10, function () {
                _this2.particleSystem.stop();
            });
        }

        /**
         * Fire the weapon if possible.
         * The mesh is animated and some particles are emitted.
         */

    }, {
        key: "fire",
        value: function fire(pickInfo) {
            if (this.canFire) {
                if (pickInfo.hit && pickInfo.pickedMesh.name === "target") {
                    pickInfo.pickedMesh.explode();
                } else {
                    var b = BABYLON.Mesh.CreateBox("box", 0.1, this.game.scene);
                    b.position = pickInfo.pickedPoint.clone();
                }
                this.animate();
                this.canFire = false;
            } else {
                // Nothing to do : cannot fire
            }
        }
    }]);

    return Weapon;
}();

;



// The function onload is loaded when the DOM has been loaded

function gallery(options) {
    new Game('canvas', options);
}

var Game = function () {
    function Game(canvasId, options) {
        var _this = this;

        _classCallCheck(this, Game);

        var canvas = document.getElementById(canvasId);
        var engine = new BABYLON.Engine(canvas, true);
        this.scene = this._initScene(engine);
        this.engine = engine;

        this.loader = new BABYLON.AssetsManager(this.scene);

        // An array containing the loaded assets
        this.assets = {};
        window.game = this;
        this.debugInfo = options.debugInfo;
        this.memory = options.memory;
        this.paintings = options.paintings;
        this.remainingTime = options.remainingTime;
        this.size = 500;

        var meshTask = this.loader.addMeshTask("gun", "", "./assets/", "gun.babylon");
        meshTask.onSuccess = function (task) {
            _this._initMesh(task);
        };

        var cTask = this.loader.addMeshTask("cat", "", "./assets/particles/", "SSAOcat.babylon");
        cTask.onSuccess = function (task) {
            _this._initMesh(task);
        };

        this.loader.onFinish = function (tasks) {

            // Player and arena creation when the loading is finished
            var player = new Player(_this);
            var arena = new Arena(_this);

            //loop
            engine.runRenderLoop(function () {
                _this.scene.render();
            });

            window.addEventListener("keyup", function (evt) {
                _this.handleUserInput(evt.keyCode);
            });
        };

        this.loader.load();

        // Resize the babylon engine when the window is resized
        window.addEventListener("resize", function () {
            if (engine) {
                engine.resize();
            }
        }, false);
    }

    _createClass(Game, [{
        key: "_initScene",

        /**
         * Init the environment of the game / skybox, camera, ...
         */
        value: function _initScene(engine) {

            var scene = new BABYLON.Scene(engine);
            // Camera attached to the canvas
            //        var camera = new BABYLON.ArcRotateCamera("Camera", 0, Math.PI/5, 10, BABYLON.Vector3.Zero(), scene);
            //        camera.maxZ = 1000;
            //        camera.attachControl(engine.getRenderingCanvas());
            //
            axis(scene, 5);

            // Update the scene background color
            scene.clearColor = new BABYLON.Color3(0.8, 0.8, 0.8);

            // Hemispheric light to light the scene
            new BABYLON.HemisphericLight("hemi", new BABYLON.Vector3(1, 2, 1), scene);

            // Skydome
            // var skybox = BABYLON.Mesh.CreateSphere("skyBox", 50, 1000, scene);
            // skybox.layerMask = 2;

            // The sky creation
            BABYLON.Engine.ShadersRepository = "shaders/";

            var shader = new BABYLON.ShaderMaterial("gradient", scene, "gradient", {});
            shader.setFloat("offset", 200);
            shader.setColor3("topColor", BABYLON.Color3.FromInts(0, 119, 255));
            shader.setColor3("bottomColor", BABYLON.Color3.FromInts(240, 240, 255));
            shader.backFaceCulling = false;

            this.clouds = ['', '', '', '', '', '', ''].map(function (e, g) {
                var cloud = new BABYLON.Mesh.CreateSphere("sphere_" + g, 10, 52, scene);
                var cloudMaterial = new BABYLON.StandardMaterial("sphere_material_" + g, scene);
                cloudMaterial.diffuseTexture = new BABYLON.Texture('assets/particles/skybox.jpg', scene);
                cloud.material = cloudMaterial;
                cloud.position.y = 292;
                cloud.position.x = (Math.random() * 180 - 90) * 7;
                cloud.position.z = (Math.random() * 180 - 90) * 7;
                return cloud;
            });

            var skybox = BABYLON.Mesh.CreateBox("skyBox", 800.0, scene);
            var skyboxMaterial = new BABYLON.StandardMaterial("skyBox", scene);
            skyboxMaterial.backFaceCulling = false;
            skyboxMaterial.reflectionTexture = new BABYLON.CubeTexture("assets/particles/skybox", scene);
            skyboxMaterial.reflectionTexture.coordinatesMode = BABYLON.Texture.SKYBOX_MODE;
            skyboxMaterial.diffuseColor = new BABYLON.Color3(0, 0, 0);
            skyboxMaterial.specularColor = new BABYLON.Color3(0, 0, 0);
            skybox.material = skyboxMaterial;

            // skybox.material = shader;

            this.skybox = skybox;
            this.xMaterial = skyboxMaterial;
            return scene;
        }

        /**
         * Handle user keyboard inputs
         * @param keycode
         */

    }, {
        key: "handleUserInput",
        value: function handleUserInput(keycode) {
            switch (keycode) {}
        }

        /**
         * Initialize a mesh once it has been loaded. Store it in the asset array and set it not visible.
         * @param task
         * @private
         */

    }, {
        key: "_initMesh",

        // user inputs
        value: function _initMesh(task) {
            this.assets[task.name] = task.loadedMeshes;
            for (var i = 0; i < task.loadedMeshes.length; i++) {
                var mesh = task.loadedMeshes[i];
                mesh.isVisible = false;
            }
        }
    }, {
        key: "dialog",
        value: function dialog(text) {
            var dynamicMaterial = new BABYLON.StandardMaterial('mat', this.game.scene);
            dynamicMaterial.diffuseTexture = new BABYLON.DynamicTexture("dynamic texture", 512, scene, true);;
            dynamicMaterial.specularColor = new BABYLON.Color3(0, 0, 0);
            dynamicMaterial.backFaceCulling = false;
            console.log('TEXT', text);
        }
    }]);

    return Game;
}();

;

/**
 * Draw axis on the scene
 * @param scene The scene where the axis will be displayed
 * @param size The size of each axis.
 */
var axis = function axis(scene, size) {
    //X axis
    var x = BABYLON.Mesh.CreateCylinder("x", size, 0.1, 0.1, 6, scene, false);
    x.material = new BABYLON.StandardMaterial("xColor", scene);
    x.material.diffuseColor = new BABYLON.Color3(1, 0, 0);
    x.position = new BABYLON.Vector3(size / 2, 0, 0);
    x.rotation.z = Math.PI / 2;

    //Y axis
    var y = BABYLON.Mesh.CreateCylinder("y", size, 0.1, 0.1, 6, scene, false);
    y.material = new BABYLON.StandardMaterial("yColor", scene);
    y.material.diffuseColor = new BABYLON.Color3(0, 1, 0);
    y.position = new BABYLON.Vector3(0, size / 2, 0);

    //Z axis
    var z = BABYLON.Mesh.CreateCylinder("z", size, 0.1, 0.1, 6, scene, false);
    z.material = new BABYLON.StandardMaterial("zColor", scene);
    z.material.diffuseColor = new BABYLON.Color3(0, 0, 1);
    z.position = new BABYLON.Vector3(0, 0, size / 2);
    z.rotation.x = Math.PI / 2;
};



// require("babel-polyfill");

// import request from "request";

var Simulator = function () {
    function Simulator() {
        _classCallCheck(this, Simulator);

        this.canvas = document.getElementById('canvas');
        // this.ctx2d = this.canvas.getContext('2d');
        // this.ctx3d = this.canvas.getContext('3d');
        this.ctx = this.canvas.getContext('webgl');
    }

    //* the game loop
    // load scenes and run them in subfunctions

    _createClass(Simulator, [{
        key: 'game',
        value: function game() {
            return regeneratorRuntime.async(function game$(_context) {
                while (1) {
                    switch (_context.prev = _context.next) {
                        case 0:
                            _context.next = 2;
                            return regeneratorRuntime.awrap(this.intro());

                        case 2:
                        case 'end':
                            return _context.stop();
                    }
                }
            }, null, this);
        }

        //* just to feck with ya

    }, {
        key: 'foo',

        // await this.bar();
        value: function foo() {
            return regeneratorRuntime.async(function foo$(_context2) {
                while (1) {
                    switch (_context2.prev = _context2.next) {
                        case 0:
                        case 'end':
                            return _context2.stop();
                    }
                }
            }, null, this);
        }

        //* a real function called bar, hold my jimmies

    }, {
        key: 'bar',
        value: function bar() {
            return regeneratorRuntime.async(function bar$(_context3) {
                while (1) {
                    switch (_context3.prev = _context3.next) {
                        case 0:
                        case 'end':
                            return _context3.stop();
                    }
                }
            }, null, this);
        }
    }, {
        key: 'intro',
        value: function intro() {
            var z, plovdiv, year, y, texts, u, l;
            return regeneratorRuntime.async(function intro$(_context4) {
                while (1) {
                    switch (_context4.prev = _context4.next) {
                        case 0:
                            z = $('#z');
                            // this.ctx2d.font = "72px serif";
                            // this.ctx2d.strokeText("H E L L O", 10, 80);

                            plovdiv = $('#plovdiv');
                            year = $('#year');
                            y = [['PLOVDIV', 2015], ['PLOVD V', 2016], ['PLOV  V', 2017], ['PLOV IV', 2018], [' LOVDIV', 2019], ['P?????V', 2019], ['    DIV', 2020], ['      _', 2020]];
                            texts = ['Something happened with plovdiv', 'Something weird and bizarre', 'I didn\'t want it to happen', 'But life is weird'];
                            u = 0;
                            l = setInterval(function () {
                                if (u < 7) {
                                    // plovdiv.html(y[u][0]);

                                    var l = y[u][0];
                                    $('#plovdiv .p').html(l[0]);
                                    $('#plovdiv .l').html(l[1]);
                                    $('#plovdiv .o').html(l[2]);
                                    $('#plovdiv .v').html(l[3]);
                                    $('#plovdiv .d').html(l[4]);
                                    $('#plovdiv .i').html(l[5]);
                                    $('#plovdiv .v2').html(l[6]);
                                    if (u % 2 == 0) {
                                        year.css('color', 'red');
                                    } else {
                                        year.css('color', 'blue');
                                    }
                                    year.html(y[u][1]);

                                    u += 1;
                                } else {
                                    clearInterval(l);
                                    setTimeout(function () {
                                        var l = y[7][0];
                                        $('#plovdiv .p').html(l[0]);
                                        $('#plovdiv .l').html(l[1]);
                                        $('#plovdiv .o').html(l[2]);
                                        $('#plovdiv .v').html(l[3]);
                                        $('#plovdiv .d').html(l[4]);
                                        $('#plovdiv .i').html(l[5]);
                                        $('#plovdiv .v2').html(l[6]);
                                        year.html(y[7][1]);
                                        var v = 0;
                                        var w = setInterval(function () {
                                            if (v < texts.length) {
                                                $('#text').html(texts[v]);
                                                $('#zz img')[0].src = 'assets/particles/zz_' + v + '.jpg';
                                                v++;
                                            } else {
                                                clearInterval(w);
                                                z.hide();
                                                $('#canvas').show();
                                                $('#viseur').show();
                                                gallery(window.plovdivOptions);
                                            }
                                        }, 10000);
                                    }, 2000);
                                }
                            }, 3000);

                        case 7:
                        case 'end':
                            return _context4.stop();
                    }
                }
            }, null, this);
        }
    }, {
        key: 'end',
        value: function end(reason) {
            var w;
            return regeneratorRuntime.async(function end$(_context5) {
                while (1) {
                    switch (_context5.prev = _context5.next) {
                        case 0:
                            $('#canvas').hide();
                            w = 0;

                            setInterval(function () {
                                $('#z').html(w % 2 == 0 ? 'PLOV DIV' : 'PLOV SIV');
                                w += 1;
                            }, 2000);

                        case 3:
                        case 'end':
                            return _context5.stop();
                    }
                }
            }, null, this);
        }
    }]);

    return Simulator;
}();

function main() {
    return regeneratorRuntime.async(function main$(_context6) {
        while (1) {
            switch (_context6.prev = _context6.next) {
                case 0:
                    window.simulator = new Simulator();
                    window.simulator.game();

                case 2:
                case 'end':
                    return _context6.stop();
            }
        }
    }, null, this);
}

document.addEventListener("DOMContentLoaded", function () {
    main(); // gallery(window.plovdivOptions);
}, false);

