class Arena {
    constructor(game) {
        this.game = game;

        // The ground
        var ground = BABYLON.Mesh.CreateGround("ground",  this.game.size,  this.game.size, 2, this.game.scene);
        var floorIsSlava = new BABYLON.StandardMaterial("floorIsSlava", this.game.scene);
        floorIsSlava.diffuseTexture = new BABYLON.Texture("assets/particles/floorIsSlava.jpg", this.game.scene);
        floorIsSlava.diffuseTexture.uScale = 5.0;
        floorIsSlava.diffuseTexture.vScale = 5.0;
        floorIsSlava.backFaceCulling = false;

        ground.material = floorIsSlava
        this._deactivateSpecular(ground);
        ground.checkCollisions = true;


        this.game.eggs = [
            new PaintingEgg(this.game,
                    this.game.paintings[0],
                    this._randomNumber(-this.game.size/2, this.game.size/2),
                    this._randomNumber(-this.game.size/2, this.game.size/2))].concat(
                    ['', '', '', ''].map((u, v) => {
                        return {
                            painting: this.game.paintings[v + 1],
                            posX: this._randomNumber(-this.game.size/2, this.game.size/2),
                            posZ: this._randomNumber(-this.game.size/2, this.game.size/2)
                        }
                    }));
        this.game.eggsCracked = 0;

        new MagritteMan('pigeon', this.game);
        for(let u = 0;u < 10; u++) {
            new MagritteMan('pigeon', this.game);
            new MagritteMan('apple', this.game);
        }
        setInterval(() => {
            new MagritteMan('pigeon', this.game);
            new MagritteMan('apple', this.game);
        }, 10000);

        
        var mm = new BABYLON.FreeCamera("minimap", new BABYLON.Vector3(0,100,0), this.game.scene);
        mm.layerMask = 1;
        mm.setTarget(new BABYLON.Vector3(0.1,0.1,0.1));
        mm.mode = BABYLON.Camera.ORTHOGRAPHIC_CAMERA;

        mm.orthoLeft = -this.game.size/2;
        mm.orthoRight = this.game.size/2;
        mm.orthoTop =  this.game.size/2;
        mm.orthoBottom = -this.game.size/2;

        mm.rotation.x = Math.PI/2;

        var xstart = 0.8,
            ystart = 0.75;
        var width = 0.99-xstart,
            height = 1-ystart;

        mm.viewport = new BABYLON.Viewport(
            xstart,
            ystart,
            width,
            height
        );
        this.game.scene.activeCameras.push(mm);
    };

    /**
     * Generates a random number between min and max
     * @param min
     * @param max
     * @returns {number}
     * @private
     */
    _randomNumber(min, max) {
        if (min == max) {
            return (min);
        }
        var random = Math.random();
        return ((random * (max - min)) + min);
    }

    _deactivateSpecular(mesh) {
        if (!mesh.material) {
            mesh.material = new BABYLON.StandardMaterial(mesh.name+"mat", this.game.scene);
        }
        mesh.material.specularColor = BABYLON.Color3.Black();
    }
};
